FileCompressionType

Struct FileCompressionType 

pub struct FileCompressionType { /* private fields */ }
Readable file compression type

Implementations

Source

impl FileCompressionType

FileCompressionType implementation

pub const GZIP: Self

Gzip-ed file

pub const BZIP2: Self

Bzip2-ed file

pub const XZ: Self

Xz-ed file (liblzma)

pub const ZSTD: Self

Zstd-ed file

pub const UNCOMPRESSED: Self

Uncompressed file

pub fn get_variant(&self) -> &CompressionTypeVariant

Read only access to self.variant

pub const fn is_compressed(&self) -> bool

The file is compressed or not

pub fn convert_to_compress_stream<'a>( &self, s: BoxStream<'a, Result<Bytes>>, ) -> Result<BoxStream<'a, Result<Bytes>>>

Given a Stream, create a Stream which data are compressed with FileCompressionType.

pub fn convert_async_writer( &self, w: BufWriter, ) -> Result<Box<dyn AsyncWrite + Send + Unpin>>

Wrap the given BufWriter so that it performs compressed writes according to this FileCompressionType using the default compression level.

pub fn convert_async_writer_with_level( &self, w: BufWriter, compression_level: Option<u32>, ) -> Result<Box<dyn AsyncWrite + Send + Unpin>>

Wrap the given BufWriter so that it performs compressed writes according to this FileCompressionType.

If compression_level is Some, the encoder will use the specified compression level. If None, the default level for each algorithm is used.

pub fn convert_stream<'a>( &self, s: BoxStream<'a, Result<Bytes>>, ) -> Result<BoxStream<'a, Result<Bytes>>>

Given a Stream, create a Stream which data are decompressed with FileCompressionType.

pub fn convert_read<T: Read + Send + 'static>( &self, r: T, ) -> Result<Box<dyn Read + Send>>

Given a Read, create a Read which data are decompressed with FileCompressionType.