#[non_exhaustive]pub struct FileMeta {
pub statistics: Statistics,
pub ordering: Option<LexOrdering>,
}Expand description
Metadata fetched from a file, including statistics and ordering.
This struct is returned by FileFormat::infer_stats_and_ordering to
provide all metadata in a single read, avoiding duplicate I/O operations.
Fields (Non-exhaustive)
This struct is marked as non-exhaustive
Non-exhaustive structs could have additional fields added in future. Therefore, non-exhaustive structs cannot be constructed in external crates using the traditional
Struct { .. } syntax; cannot be matched against without a wildcard ..; and struct update syntax will not work.statistics: StatisticsStatistics for the file (row counts, byte sizes, column statistics).
ordering: Option<LexOrdering>The ordering (sort order) of the file, if known.
Implementations
Sourceimpl FileMeta
impl FileMeta
Sourcepub fn new(statistics: Statistics) -> Self
pub fn new(statistics: Statistics) -> Self
Creates a new FileMeta with the given statistics and no ordering.
Sourcepub fn with_ordering(self, ordering: Option<LexOrdering>) -> Self
pub fn with_ordering(self, ordering: Option<LexOrdering>) -> Self
Sets the ordering for this file metadata.