FileScanConfig to configure scanning of possibly partitioned
file sources.
Structs
- File
Scan Config FileScanConfigrepresents scanning data from a group of files- File
Scan Config Builder - A builder for
FileScanConfig’s.
Functions
- output_
partitioning_ from_ partition_ fields - Builds output partitioning over
partition_cols(resolved to their indices inschema) withpartition_countpartitions. ReturnsNonewhen there are no partition columns. Callers use this to declare the output partitioning of a scan whose file groups are organized by partition column values. - wrap_
partition_ type_ in_ dict - Convert type to a type suitable for use as a
ListingTablepartition column. ReturnsDictionary(UInt16, val_type), which is a reasonable trade off between a reasonable number of partition values and space efficiency. - wrap_
partition_ value_ in_ dict - Convert a
ScalarValueof partition columns to a type, as described in the documentation ofwrap_partition_type_in_dict, which can wrap the types.