FileOutputMode

Enum FileOutputMode 

pub enum FileOutputMode {
    Automatic,
    SingleFile,
    Directory,
}
Determines how FileSink output paths are interpreted.

Variants



Automatic

Infer output mode from the output URL (for example, by extension / trailing /).



SingleFile

Write to a single output file at the exact output path.



Directory

Write to a directory under the output path with generated filenames.

Implementations

Source

impl FileOutputMode

pub fn single_file_output(self, base_output_path: &ListingTableUrl) -> bool

Resolve this mode into a single_file_output boolean for the demuxer.