Module file_stream

Module file_stream 

A generic stream over file format readers that can be used by any file format that read its files from start to end.

Note: Most traits here need to be marked Sync + Send to be compliant with the SendableRecordBatchStream trait.

Structs

FileStream
A stream that iterates record batch by record batch, file over file.
FileStreamBuilder
Builder for constructing a FileStream.
FileStreamMetrics
Metrics for FileStream
StartableTime
A timer that can be started and stopped.

Enums

OnError
Describes the behavior of the FileStream if file opening or scanning fails

Traits

FileOpener
Generic API for opening a file using an ObjectStore and resolving to a stream of RecordBatch

Type Aliases

FileOpenFuture
A fallible future that resolves to a stream of RecordBatch