Morselizer

Trait Morselizer 

pub trait Morselizer:
    Send
    + Sync
    + Debug {
    // Required method
    fn plan_file(&self, file: PartitionedFile) -> Result<Box<dyn MorselPlanner>>;
}
A Morselizer takes a single PartitionedFile and creates the initial planner for that file.

This is the entry point for morsel driven I/O.

Required Methods

fn plan_file(&self, file: PartitionedFile) -> Result<Box<dyn MorselPlanner>>

Return the initial MorselPlanner for this file.

Morselizing a file may involve CPU work, such as parsing parquet metadata and evaluating pruning predicates. It should NOT do any I/O work, such as reading from the file. Any needed I/O should be done using MorselPlan::with_pending_planner.

Dyn Compatibility

This trait is dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety".

Implementors