pub struct DefaultSchemaAdapterFactory;DefaultSchemaAdapterFactory has been removed. Use PhysicalExprAdapterFactory instead. See upgrading.md for more details.
Deprecated: Default SchemaAdapterFactory for mapping schemas.
This struct has been removed.
Use PhysicalExprAdapterFactory instead to customize scans via
FileScanConfigBuilder, i.e. if you had implemented a custom SchemaAdapter
and passed that into FileScanConfigBuilder / ParquetSource.
Use BatchAdapter if you want to map a stream of RecordBatches
between one schema and another, i.e. if you were calling SchemaMapper::map_batch manually.
See upgrading.md for more details.
Implementations
Sourceimpl DefaultSchemaAdapterFactory
impl DefaultSchemaAdapterFactory
Sourcepub fn from_schema(table_schema: SchemaRef) -> Box<dyn SchemaAdapter>
๐Deprecated since 52.0.0: DefaultSchemaAdapterFactory has been removed. Use PhysicalExprAdapterFactory instead. See upgrading.md for more details.
pub fn from_schema(table_schema: SchemaRef) -> Box<dyn SchemaAdapter>
DefaultSchemaAdapterFactory has been removed. Use PhysicalExprAdapterFactory instead. See upgrading.md for more details.
Deprecated: Create a new factory for mapping batches from a file schema to a table schema.