SchemaAdapter

Trait SchemaAdapter 

pub trait SchemaAdapter: Send + Sync {
    // Required methods
    fn map_column_index(
        &self,
        index: usize,
        file_schema: &Schema,
    ) -> Option<usize>;
    fn map_schema(
        &self,
        file_schema: &Schema,
    ) -> Result<(Arc<dyn SchemaMapper>, Vec<usize>)>;
}
๐Ÿ‘ŽDeprecated since 52.0.0:

SchemaAdapter has been removed. Use PhysicalExprAdapterFactory instead. See upgrading.md for more details.

Deprecated: Creates SchemaMappers to map file-level RecordBatches to a table schema.

This trait has been removed. Use PhysicalExprAdapterFactory instead. See upgrading.md for more details.

Required Methods

fn map_column_index(&self, index: usize, file_schema: &Schema) -> Option<usize>

๐Ÿ‘ŽDeprecated since 52.0.0:

SchemaAdapter has been removed. Use PhysicalExprAdapterFactory instead. See upgrading.md for more details.

Map a column index in the table schema to a column index in a particular file schema.

fn map_schema( &self, file_schema: &Schema, ) -> Result<(Arc<dyn SchemaMapper>, Vec<usize>)>

๐Ÿ‘ŽDeprecated since 52.0.0:

SchemaAdapter has been removed. Use PhysicalExprAdapterFactory instead. See upgrading.md for more details.

Creates a mapping for casting columns from the file schema to the table schema.

Dyn Compatibility

This trait is dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety".

Implementors