pub struct DataSinkExec { /* private fields */ }Expand description
Execution plan for writing record batches to a DataSink
Returns a single row with the number of values written
Implementations
Sourceimpl DataSinkExec
impl DataSinkExec
Sourcepub fn new(
input: Arc<dyn ExecutionPlan>,
sink: Arc<dyn DataSink>,
sort_order: Option<LexRequirement>,
) -> Self
pub fn new( input: Arc<dyn ExecutionPlan>, sink: Arc<dyn DataSink>, sort_order: Option<LexRequirement>, ) -> Self
Create a plan to write to sink
Note: DataSinkExec requires its input to have a single partition.
If the input has multiple partitions, the physical optimizer will
automatically insert a Merge-related operator to merge them.
If you construct PhysicalPlan without going through the physical optimizer,
you must ensure that the input has a single partition.
Sourcepub fn input(&self) -> &Arc<dyn ExecutionPlan> ⓘ
pub fn input(&self) -> &Arc<dyn ExecutionPlan> ⓘ
Input execution plan
Sourcepub fn sort_order(&self) -> &Option<LexRequirement>
pub fn sort_order(&self) -> &Option<LexRequirement>
Optional sort order for output data
Sourcepub fn encode_sort_order(
&self,
ctx: &ExecutionPlanEncodeCtx<'_>,
) -> Result<Option<PhysicalSortExprNodeCollection>>
Available on crate feature proto only.
pub fn encode_sort_order( &self, ctx: &ExecutionPlanEncodeCtx<'_>, ) -> Result<Option<PhysicalSortExprNodeCollection>>
proto only.Encode the optional sink ordering for a protobuf plan node.
Sourcepub fn decode_sort_order(
collection: Option<&PhysicalSortExprNodeCollection>,
ctx: &ExecutionPlanDecodeCtx<'_>,
schema: &Schema,
) -> Result<Option<LexRequirement>>
Available on crate feature proto only.
pub fn decode_sort_order( collection: Option<&PhysicalSortExprNodeCollection>, ctx: &ExecutionPlanDecodeCtx<'_>, schema: &Schema, ) -> Result<Option<LexRequirement>>
proto only.Decode the optional sink ordering from a protobuf plan node.