Helper struct to manage table schemas with partition columns
Structs
- Table
Schema - The overall schema for potentially partitioned data sources.
- Table
Schema Builder - Builder for
TableSchema.
Helper struct to manage table schemas with partition columns
TableSchema.