pub fn get_writer_schema(config: &FileSinkConfig) -> Arc<Schema> ⓘExpand description
Converts table schema to writer schema, which may differ in the case of hive style partitioning where some columns are removed from the underlying files.
pub fn get_writer_schema(config: &FileSinkConfig) -> Arc<Schema> ⓘConverts table schema to writer schema, which may differ in the case of hive style partitioning where some columns are removed from the underlying files.