pub trait Morsel: Send + Debug {
// Required method
fn into_stream(self: Box<Self>) -> BoxStream<'static, Result<RecordBatch>>;
}Expand description
A Morsel of work ready to resolve to a stream of RecordBatches.
This represents a single morsel of work that is ready to be processed. It
has all data necessary (does not need any I/O) and is ready to be turned
into a stream of RecordBatches for processing by the execution engine.
Required Methods
Sourcefn into_stream(self: Box<Self>) -> BoxStream<'static, Result<RecordBatch>>
fn into_stream(self: Box<Self>) -> BoxStream<'static, Result<RecordBatch>>
Consume this morsel and produce a stream of RecordBatches for processing.
Note: This may do CPU work to decode already-loaded data, but should not do any I/O work such as reading from the file.
Dyn Compatibility
This trait is dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".