pub struct SchemaMapping { /* private fields */ }๐Deprecated since 52.0.0: SchemaMapping has been removed. Use PhysicalExprAdapterFactory instead. See upgrading.md for more details.
Deprecated: The SchemaMapping struct held a mapping from the file schema to the table schema.
This struct has been removed.
Use PhysicalExprAdapterFactory instead to customize scans via
FileScanConfigBuilder, i.e. if you had implemented a custom SchemaAdapter
and passed that into FileScanConfigBuilder / ParquetSource.
Use BatchAdapter if you want to map a stream of RecordBatches
between one schema and another, i.e. if you were calling SchemaMapper::map_batch manually.
See upgrading.md for more details.