pub struct DataSinkExec { /* private fields */ }Expand description
Execution plan for writing record batches to a DataSink
Returns a single row with the number of values written
Implementations
Sourceimpl DataSinkExec
impl DataSinkExec
Sourcepub fn new(
input: Arc<dyn ExecutionPlan>,
sink: Arc<dyn DataSink>,
sort_order: Option<LexRequirement>,
) -> Self
pub fn new( input: Arc<dyn ExecutionPlan>, sink: Arc<dyn DataSink>, sort_order: Option<LexRequirement>, ) -> Self
Create a plan to write to sink
Note: DataSinkExec requires its input to have a single partition.
If the input has multiple partitions, the physical optimizer will
automatically insert a Merge-related operator to merge them.
If you construct PhysicalPlan without going through the physical optimizer,
you must ensure that the input has a single partition.
Sourcepub fn input(&self) -> &Arc<dyn ExecutionPlan>
pub fn input(&self) -> &Arc<dyn ExecutionPlan>
Input execution plan
Sourcepub fn sort_order(&self) -> &Option<LexRequirement>
pub fn sort_order(&self) -> &Option<LexRequirement>
Optional sort order for output data